/*--------------------------------------------------------------------------
 * Copyright (C) 1992,1993,1994,1995,1996 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 */
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "lineun.h"
#include "uline.h"

//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TLineForm *LineForm;
//---------------------------------------------------------------------------
__fastcall TLineForm::TLineForm(TComponent* Owner)
   : TForm(Owner)
{
}

//---------------------------------------------------------------------------
// Create event, initialize program
//
void __fastcall TLineForm::FormCreate(TObject *Sender)
{
   char ReturnString[256];
   HKEY GlobKey;
   DWORD dwType,size;

   // set name/version
   sprintf(m_zsAppName,"DS2480 Line Control 32 - Version 2.00");
   Caption = m_zsAppName;

   /* read the default PortNum and PortType from the registry */
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
         "Software\\Dallas Semiconductor\\iButton TMEX\\3.00",0,
         KEY_READ,&GlobKey) == ERROR_SUCCESS)
   {
      /* attempt to read the PortNum */
      size = 255;
      PortNum = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortNum",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortNum = (short)atoi(ReturnString);
      }
      /* attempt to read the PortType */
      size = 255;
      PortType = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortType",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortType = (short)atoi(ReturnString);
      }

      // close the key
      RegCloseKey(GlobKey);
   }
   else
   {
      MessageBeep(MB_OK);
      Application->MessageBox("TMEX has not be correctly installed, missing default port!",m_zsAppName,
                               MB_OK | MB_ICONSTOP);
      Application->Terminate();
   }

   // set the default port string
   DefaultPort->Caption = "Type:" + IntToStr(PortType) +
        ", Number:" + IntToStr(PortNum);

   // set to info page     
   PageControl->ActivePage = PageControl->Pages[0];

   // read the current line settings (for normal line)
   NoLine = FALSE;
   if (!ReadLineSettings())
   {
      MessageBeep(MB_OK);
      Application->MessageBox("The Default port does support DS2480 line settings, change your default port!",m_zsAppName,
                               MB_OK | MB_ICONSTOP);
      NoLine = TRUE;
   }

   // Get main driver version
   Get_Version(ReturnString);
   MainDriver->Caption = ReturnString;
   // Get type driver version
   TMGetTypeVersion(PortType,ReturnString);
   TypeDriver->Caption = ReturnString;
}

//---------------------------------------------------------------------------
// Read the line settings for the current port type
//
bool __fastcall TLineForm::ReadLineSettings(void)
{
   LineType Line;
   short i,rt=0,val,temp,cnt;

   // get a session
   if (AquireSession())
   {
      // perform a setup
      val = TMSetup(SessionHandle);
      if ((val == 1) || (val == 2))
      {
         // read line settings
         sprintf(Line.ID,"DS2480");
         Line.Operation = READ_FUNCTION;
         if (TMLine(SessionHandle,&Line) == 1)
         {
            // set the values
            PDSRCNormal->ItemIndex = (Line.NormalDefault.PDSRC >> 1) & 0x07;
            W1LTNormal->ItemIndex = (Line.NormalDefault.W1LT >> 1) & 0x07;
            DSOWORTNormal->ItemIndex = (Line.NormalDefault.DSOWORT >> 1) & 0x07;

            PDSRCRelaxed->ItemIndex = (Line.RelaxedDefault.PDSRC >> 1) & 0x07;
            W1LTRelaxed->ItemIndex = (Line.RelaxedDefault.W1LT >> 1) & 0x07;
            DSOWORTRelaxed->ItemIndex = (Line.RelaxedDefault.DSOWORT >> 1) & 0x07;

            NormalSearchBaud->ItemIndex = (Line.NormalDefault.SearchBaud >> 1) & 0x03;
            NormalBitBaud->ItemIndex = (Line.NormalDefault.BitBaud >> 1) & 0x03;
            NormalByteBaud->ItemIndex = (Line.NormalDefault.ByteBaud >> 1) & 0x03;

            OverSearchBaud->ItemIndex = (Line.OverDriveDefault.SearchBaud >> 1) & 0x03;
            OverBitBaud->ItemIndex = (Line.OverDriveDefault.BitBaud >> 1) & 0x03;
            OverByteBaud->ItemIndex = (Line.OverDriveDefault.ByteBaud >> 1) & 0x03;

            RelaxSearchBaud->ItemIndex = (Line.RelaxedDefault.SearchBaud >> 1) & 0x03;
            RelaxBitBaud->ItemIndex = (Line.RelaxedDefault.BitBaud >> 1) & 0x03;
            RelaxByteBaud->ItemIndex = (Line.RelaxedDefault.ByteBaud >> 1) & 0x03;

            MaxBaud->ItemIndex = (Line.MXBAUD >> 1) & 0x03;

            cnt = 0;
            temp = Line.MAXSEND;
            while (temp > 0)
            {
               cnt++;
               temp >>= 1;
            }
            MaxSend->ItemIndex = cnt-2;

            cnt = 0;
            temp = Line.MINSEND;
            while (temp > 0)
            {
               cnt++;
               temp >>= 1;
            }
            MinSend->ItemIndex = cnt-2;

            rt = 1;
         }
         else
            Application->MessageBox("Error calling TMLine!",m_zsAppName,
                                  MB_OK | MB_ICONSTOP);
      }
      else
         Application->MessageBox("TMSetup failed.",m_zsAppName,
                               MB_OK | MB_ICONSTOP);

      // end the session
      TMEndSession(SessionHandle);
   }
   else
      Application->MessageBox("Could not aquire a session to read the settings!",m_zsAppName,
                               MB_OK | MB_ICONSTOP);


   return rt;
}

//---------------------------------------------------------------------------
// Attempt to aquire a session.  Give up after 3 secconds
//
bool __fastcall TLineForm::AquireSession(void)
{
   short cnt=0;

   do
   {
      SessionHandle = TMExtendedStartSession(PortNum, PortType, NULL);
      Sleep(10);
   }
   while ((SessionHandle == 0) && (cnt++ < 300));

   return (SessionHandle > 0);
}

//---------------------------------------------------------------------------
void __fastcall TLineForm::LineComboChange(TObject *Sender)
{
   WriteLineSettings();
   ReadLineSettings();
}

//---------------------------------------------------------------------------
// Write the line settings for the current port type
//
bool __fastcall TLineForm::WriteLineSettings(void)
{
   LineType Line;
   short rt=0,cnt,i;

   // get a session
   if (AquireSession())
   {
      // read line settings
      sprintf(Line.ID,"DS2480");
      Line.Operation = READ_FUNCTION;
      if (TMLine(SessionHandle,&Line) == 1)
      {
         // set the values
         Line.NormalDefault.PDSRC = PDSRCNormal->ItemIndex << 1;
         Line.NormalDefault.W1LT = W1LTNormal->ItemIndex << 1;
         Line.NormalDefault.DSOWORT = DSOWORTNormal->ItemIndex << 1;

         Line.RelaxedDefault.PDSRC = PDSRCRelaxed->ItemIndex << 1;
         Line.RelaxedDefault.W1LT = W1LTRelaxed->ItemIndex << 1;
         Line.RelaxedDefault.DSOWORT = DSOWORTRelaxed->ItemIndex << 1;

         Line.NormalDefault.SearchBaud = NormalSearchBaud->ItemIndex << 1;
         Line.NormalDefault.BitBaud = NormalBitBaud->ItemIndex << 1;
         Line.NormalDefault.ByteBaud = NormalByteBaud->ItemIndex << 1;

         Line.OverDriveDefault.SearchBaud = OverSearchBaud->ItemIndex << 1;
         Line.OverDriveDefault.BitBaud = OverBitBaud->ItemIndex << 1;
         Line.OverDriveDefault.ByteBaud = OverByteBaud->ItemIndex << 1;

         Line.RelaxedDefault.SearchBaud = RelaxSearchBaud->ItemIndex << 1;
         Line.RelaxedDefault.BitBaud = RelaxBitBaud->ItemIndex << 1;
         Line.RelaxedDefault.ByteBaud = RelaxByteBaud->ItemIndex << 1;

         Line.MXBAUD = MaxBaud->ItemIndex << 1;

         cnt = 2;
         for (i = 0; i < MaxSend->ItemIndex; i++)
            cnt *= 2;
         Line.MAXSEND = cnt;

         cnt = 2;
         for (i = 0; i < MinSend->ItemIndex; i++)
            cnt *= 2;
         Line.MINSEND = cnt;

         Line.Operation = WRITE_FUNCTION;
         // write them back
         if (TMLine(SessionHandle,&Line) == 1)
            rt = 1;
         else
            Application->MessageBox("Error calling TMLine!",m_zsAppName,
                               MB_OK | MB_ICONSTOP);

      }
      else
         Application->MessageBox("Error calling TMLine!",m_zsAppName,
                               MB_OK | MB_ICONSTOP);

      // end the session
      TMEndSession(SessionHandle);
   }
   else
      Application->MessageBox("Could not aquire a session to write the settings!",m_zsAppName,
                               MB_OK | MB_ICONSTOP);

   return rt;
}

//---------------------------------------------------------------------------
// change page event
//
void __fastcall TLineForm::PageControlChanging(TObject *Sender,
   bool &AllowChange)
{
   if (NoLine)
   {
      Application->MessageBox("The Default port does support DS2480 line settings, change your default port!",m_zsAppName,
                               MB_OK | MB_ICONSTOP);
      AllowChange = FALSE;
   }
}
//---------------------------------------------------------------------------
